@@ -110,8 +110,13 @@ _gcry_ecc_ecdsa_sign (gcry_mpi_t input, gcry_mpi_t k_supplied, mpi_ec_t ec,
                       (hashalgo == GCRY_MD_SHAKE128
                        || hashalgo == GCRY_MD_SHAKE256))
                     {
-                      rc = GPG_ERR_DIGEST_ALGO;
-                      goto leave;
+                      if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
+                        {
+                          rc = GPG_ERR_DIGEST_ALGO;
+                          goto leave;
+                        }
+                      else
+                        fips_service_indicator_mark_non_compliant ();
                     }
 
                   /* Use Pornin's method for deterministic DSA.  If this
