@@ -4,7 +4,7 @@
  * This file is part of Libgcrypt.
  *
  * Libgcrypt is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser general Public License as
+ * it under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
@@ -22,9 +22,10 @@
 
 /* Context types as used in struct gcry_context.  */
 #define CONTEXT_TYPE_EC 1  /* The context is used with EC functions.  */
-#define CONTEXT_TYPE_RANDOM_OVERRIDE 2  /* Used with pubkey functions.  */
+#define CONTEXT_TYPE_SINGLE_DATA   2  /* Used with pubkey functions.  */
 
-gcry_ctx_t _gcry_ctx_alloc (int type, size_t length, void (*deinit)(void*));
+gcry_ctx_t _gcry_ctx_alloc (int type, size_t length, void (*deinit)(void*),
+                            gcry_ctx_t next);
 void *_gcry_ctx_get_pointer (gcry_ctx_t ctx, int type);
 void *_gcry_ctx_find_pointer (gcry_ctx_t ctx, int type);
 
